@@ -1,6 +1,6 @@
 package Net::Amazon::S3::Client;
 # ABSTRACT: An easy-to-use Amazon S3 client
-$Net::Amazon::S3::Client::VERSION = '0.96';
+$Net::Amazon::S3::Client::VERSION = '0.97';
 use Moose 0.85;
 use HTTP::Status qw(status_message);
 use MooseX::StrictConstructor 0.16;
@@ -31,6 +31,13 @@ has error_handler => (
 	default => sub { $_[0]->error_handler_class->new (s3 => $_[0]->s3) },
 );
 
+has bucket_class => (
+	is          => 'ro',
+	init_arg    => undef,
+	lazy        => 1,
+	default     => 'Net::Amazon::S3::Client::Bucket',
+);
+
 around BUILDARGS => sub {
 	my ($orig, $class) = (shift, shift);
 	my $args = $class->$orig (@_);
@@ -50,8 +57,6 @@ around BUILDARGS => sub {
 
 __PACKAGE__->meta->make_immutable;
 
-sub bucket_class { 'Net::Amazon::S3::Client::Bucket' }
-
 sub buckets {
 	my $self = shift;
 	my $s3   = $self->s3;
@@ -122,7 +127,7 @@ Net::Amazon::S3::Client - An easy-to-use Amazon S3 client
 
 =head1 VERSION
 
-version 0.96
+version 0.97
 
 =head1 SYNOPSIS
 
