@@ -1,6 +1,6 @@
 package Shared::Examples::Net::Amazon::S3::Client;
 # ABSTRACT: used for testing and as example
-$Shared::Examples::Net::Amazon::S3::Client::VERSION = '0.96';
+$Shared::Examples::Net::Amazon::S3::Client::VERSION = '0.97';
 use strict;
 use warnings;
 
@@ -77,6 +77,7 @@ sub expect_signed_uri {
 		qw[ with_region ],
 		qw[ with_key ],
 		qw[ with_expire_at ],
+		qw[ with_method ],
 		qw[ expect_uri ],
 		;
 
@@ -92,7 +93,9 @@ sub expect_signed_uri {
 			key     => $params{with_key},
 			expires => $params{with_expire_at},
 		)
-		->query_string_authentication_uri
+		->query_string_authentication_uri_for_method (
+			$params{with_method} || 'GET',
+		)
 		;
 
 	cmp_deeply $got, $params{expect_uri}, $title;
@@ -290,7 +293,7 @@ Shared::Examples::Net::Amazon::S3::Client - used for testing and as example
 
 =head1 VERSION
 
-version 0.96
+version 0.97
 
 =head1 AUTHOR
 
